#define RABBIT_SCORE 1
#define RABBIT_PIECE_MIN_LIFETIME_MOVES 15
#define RABBIT_PIECE_MAX_LIFETIME_MOVES 45

#include <avr/io.h>

extern "C" {
#include <../libraries/usb_debug_only/print.h>
}

#include "Snaxe.h"


GameBoard::GameBoard(pos_t width, pos_t height)
{
	this->width = width;
	this->height = height;
	firstPiece = 0;

	snakeHead = snakeTail = new SnakePart(width / 2, height / 2);
	snakeFacing = EAST;
	score = 0;
	snakeDead = false;

	snakeGrow = 1;
	moveSnake(); // grow it one square
}

GameBoard::~GameBoard()
{
	SnakePart *part = snakeHead;
	while (part != 0) {
		SnakePart *nextPart = part->next;
		delete part;
		part = nextPart;
	}
}

void GameBoard::move()
{
	// Move snake
	moveSnake();

	// if a snake is now off-grid, it dies.
	if (snakeHead->x < 0 || snakeHead->x >= width || snakeHead->y < 0 || snakeHead->y >= height) {
		print("Snake off-grid\n");
		snakeDead = true;
		return;
	}


    // if a snake occupies the position of theirs body or tail, they die

	SnakePart *part = snakeHead->next; // allow to touch their own head!
	while (part != 0) {
		if (snakeHead->x == part->x && snakeHead->y == part->y) {
			print("Snake touched tail\n");
			snakeDead = true;
			return;
		}
		part = part->next;
	}

    // if a snake occupies the position of a board piece, they die or gain the powerup
	
	Piece *lastPiece = 0;
	Piece *piece = firstPiece;
	
	while (piece != 0) {
		if (snakeHead->x == piece->x && snakeHead->y == piece->y) {

			switch (piece->type) {
			case RABBIT:
				print("Snake eats rabbit\n");
				score += RABBIT_SCORE;
				snakeGrow++;
				break;
//          case SKULL:
//            log.log(Level.FINE, "Snake " + snake + " was killed by a skull.");
//            goto killSnake;
//            break;
			}

			if (lastPiece == 0) {
				firstPiece = piece->next;
			}
			else {
				lastPiece->next = piece->next;
			}
			delete piece;
			break; // this is OK because there can't be more than one piece at a position
		}

		lastPiece = piece;
		piece = piece->next;
	}


    // Now decay old powerups

	piece = firstPiece;
	while (piece != 0) {
		piece->lifetimeRemaining--;
		piece = piece->next;
	}
		
	restartDecayPieces:
	lastPiece = 0;
	piece = firstPiece;
	while (piece != 0) {
		if (piece->lifetimeRemaining <= 0) {
			print("Piece decays\n");

			if (lastPiece == 0) {
				firstPiece = piece->next;
			}
			else {
				lastPiece->next = piece->next;
			}
			goto restartDecayPieces;
		}

		lastPiece = piece;
		piece = piece->next;
	}

    // Now add new powerups
    populateRabbits();
}

void GameBoard::moveSnake()
{
	pos_t newX = snakeHead->x;
	pos_t newY = snakeHead->y;
	switch (snakeFacing) {
	case NORTH:
		newY++;
		break;
	case EAST:
		newX++;
		break;
	case SOUTH:
		newY--;
		break;
	case WEST:
		newX--;
		break;
	default:
		return;
	}

	SnakePart *newPart = new SnakePart(newX, newY);
	newPart->next = snakeHead;
	newPart->nextPieceDir = opposite(snakeFacing);
	snakeHead->previous = newPart;
	snakeHead->prevPieceDir = snakeFacing;
	snakeHead = newPart;

	// If we're not growing, we need to kill the tail
	if (snakeGrow > 0) {
		print("Snake grows\n");
		snakeGrow--;
	}
	else {
		SnakePart *oldTail = snakeTail;
		snakeTail->previous->nextPieceDir = NODIR;
		snakeTail->previous->next = 0;
		snakeTail = snakeTail->previous;
		delete oldTail;
	}
}

void GameBoard::setFacing(Direction d)
{
	if (snakeHead->nextPieceDir != d) {
		snakeFacing = d;
	}
}


Direction GameBoard::opposite(Direction d)
{
	switch (d) {
	case NORTH:
		return SOUTH;
	case EAST:
		return WEST;
	case SOUTH:
		return NORTH;
	case WEST:
		return EAST;
	default:
		break;
	}
	return NODIR;
}
	

void GameBoard::populateRabbits()
{
	Piece *piece = firstPiece;
	while (piece != 0) {
		if (piece->type == RABBIT) {
			return; // already a rabbit on the board
		}
		piece = piece->next;
    }
	print("Rabbit needed\n");

	addPieceAtRandomUnoccupiedPoint(RABBIT, RABBIT_PIECE_MIN_LIFETIME_MOVES, RABBIT_PIECE_MAX_LIFETIME_MOVES);
}


void GameBoard::addPieceAtRandomUnoccupiedPoint(PieceType type, int minLifetime, int maxLifetime)
{
	pos_t x, y;
	uint8_t pointOk;

    for (int i = 0; i < 50; ++i) {
		x = random() % width;
		y = random() % height;
		pointOk = 1;

		// Make sure there's not an existing powerup or snake part there
		Piece *piece = firstPiece;
		while (piece != 0) {
			if (piece->x == x && piece->y == y) {
				pointOk = 0;
				break;
			}
			piece = piece->next;
		}

		SnakePart *part = snakeHead;
		while (part != 0) {
			if (part->x == x && part->y == y) {
				pointOk = 0;
				break;
			}

			part = part->next;
		}

		if (pointOk) {
			break;
		}
    }

	if (!pointOk) return; // couldn't find an unoccupied point in time


    uint8_t lifetime = minLifetime + (random() % (maxLifetime + 1 - minLifetime));
	print("Rabbit placed at ");
	phex(x); print(","); phex(y); print("\n");
	print("Rabbit lifetime is "); phex(lifetime); print("\n");

	Piece *piece = new Piece(type, x, y, lifetime);
	piece->next = firstPiece;
	firstPiece = piece;
}


void * GameBoard::operator new(size_t size) 
{ 
  return malloc(size); 
} 

void GameBoard::operator delete(void * ptr) 
{ 
  free(ptr); 
} 


void * SnakePart::operator new(size_t size) 
{ 
  return malloc(size); 
} 

void SnakePart::operator delete(void * ptr) 
{ 
  free(ptr); 
} 

void * Piece::operator new(size_t size) 
{ 
  return malloc(size); 
} 

void Piece::operator delete(void * ptr) 
{ 
  free(ptr); 
} 
