#include <stdio.h>
#include <string.h>

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#define USB_DEBUG 0

extern "C" {
#include <usb_debug_only.h>
#ifdef USB_DEBUG
#include <../libraries/usb_debug_only/print.h>
#endif
#include <../libraries/analog/analog.h>

#include "core_pins.h"
//#include "img/head.xbm"
}

#include "Dogm.h"

#include "Snaxe.h"

#define CPU_PRESCALE(n) (CLKPR = 0x80, CLKPR = (n))
#define CPU_16MHz       0x00
#define CPU_8MHz        0x01
#define CPU_4MHz        0x02
#define CPU_2MHz        0x03
#define CPU_1MHz        0x04
#define CPU_500kHz      0x05
#define CPU_250kHz      0x06
#define CPU_125kHz      0x07
#define CPU_62kHz       0x08

#define ADC_MUX_PIN_F0    0x00
#define ADC_MUX_PIN_F1    0x01
#define ADC_MUX_PIN_F2    0x02
#define ADC_MUX_PIN_F3    0x03
#define ADC_MUX_PIN_F4    0x04
#define ADC_MUX_PIN_F5    0x05
#define ADC_MUX_PIN_F6    0x06
#define ADC_MUX_PIN_F7    0x07
#define ADC_MUX_PIN_D4    0x20
#define ADC_MUX_PIN_D6    0x21
#define ADC_MUX_PIN_D7    0x22
#define ADC_MUX_PIN_B4    0x23
#define ADC_MUX_PIN_B5    0x24
#define ADC_MUX_PIN_B6    0x25


#ifdef USB_DEBUG
static void usb_debug_putstr(char *str);
#endif

void insertCoin(Dogm &dogm, const char *title, const char *subtitle);

uint16_t playGame(Dogm &dogm);

char tmpbuf[128];

int main(void)
{
	CPU_PRESCALE(CPU_16MHz);

	// Default all lines to high-impedance floating inputs
	DDRB = DDRC = DDRD = DDRF = 0;
	PORTB = PORTC = PORTD = PORTF = 0;

	// Button on F4, pull-up
	PORTF |= (1<<4);

	_init_Teensyduino_internal_();
	
	usb_init();
	
#ifdef USB_DEBUG
	// Wait for USB host to enumerate us
	while (!usb_configured()) {
	}
	
	// Wait 1 second more to allow drivers to load
	_delay_ms(1000);
#endif
	

	analogReference(ADC_REF_POWER);

	Dogm dogm(PIN_A0_DEFAULT); // address line a0 for the dogm module
	
	dogm.setInvertPixelMode(1);

//	dogm.showLibInfo();

	insertCoin(dogm, "SNAXE", "(c) 1984 David C A Croft");

	while (1) {
		uint16_t score = playGame(dogm);
		
		sprintf(tmpbuf, "Score: %d", score);
		insertCoin(dogm, "Game Over!", tmpbuf);
	}
}


uint16_t playGame(Dogm &dogm)
{
	int16_t val;
	int16_t left, right, up, down;

	GameBoard *board = new GameBoard(DOG_WIDTH/4, DOG_HEIGHT/4);

	int16_t speed;

	while (1) {
		val = adc_read(ADC_MUX_PIN_F0);
		left = val - 0x020f;
		right = 0x0209 - val;
		
		val = adc_read(ADC_MUX_PIN_F1);
		up = val - 0x0209;
		down = 0x0203 - val;

		if (left > 0 && left >= up && left >= down)
			board->setFacing(WEST);
		else if (right > 0 && right >= up && right >= down)
			board->setFacing(EAST);
		else if (up > 0 && up >= left && up >= right)
			board->setFacing(NORTH);
		else if (down > 0 && down >= left && down >= right)
			board->setFacing(SOUTH);
		
		board->move();
  
		if (board->snakeDead)
			break;
		
		dogm.start();

		do {
			SnakePart *part = board->snakeHead;
			while (part != 0) {
				dogm.setBox(part->x * 4, part->y * 4, ((part->x)+1) * 4, ((part->y)+1) *4);
				part = part->next;
			}
			
			Piece *piece = board->firstPiece;
			while (piece != 0) {
				if (piece->type == RABBIT) {
					dogm.setBox((piece->x * 4)+1, piece->y * 4, (((piece->x)+1) * 4)-1, ((piece->y)+1) *4);
					dogm.setBox(piece->x * 4, (piece->y * 4)+1, ((piece->x)+1) * 4, (((piece->y)+1) *4)-1);
				}
				piece = piece->next;
			}
			
		} while (dogm.next());

		speed = 500 - (50 * board->score);
		if (speed < 100) speed = 100;

		delay(speed);
	}
	
	uint16_t score = board->score;
	delete board;
	return score;
}


#ifdef USB_DEBUG
static void usb_debug_putstr(char *str)
{
	while ((*str) != 0) {
		usb_debug_putchar(*(str++));
	}
}
#endif


void insertCoin(Dogm &dogm, const char *title, const char *subtitle)
{
	uint8_t on = 1;

	while (1) {

		dogm.start();

		do {	
			dogm.setFont(font_9x18);
			dogm.setXY((DOG_WIDTH - dogm.getStrWidth(title)) / 2, 40);
			dogm.write(title);

			dogm.setFont(font_4x6);
			dogm.setXY((DOG_WIDTH - dogm.getStrWidth(subtitle)) / 2, 25);
			dogm.write(subtitle);

			if (on) {
				dogm.setFont(font_4x6);
				const char *str = "INSERT COIN"; // TODO PROGMEM doesn't work
				dogm.setXY((DOG_WIDTH - dogm.getStrWidth(str)) / 2, 10);
				dogm.write(str);
			}
		} while (dogm.next());

		uint32_t waitUntil = millis() + 500;
		while (millis() < waitUntil) {
			if (!(PINF & (1<<4))) {
				return;
			}
		}

		on = 1-on;
	}
}
